%{
Solve a finite time horizon model of car buying with quasi-hyperbolic 
discounting using backwards induction
Include all four vehicle types

3/14/2011

Garth Heutel
gaheutel@uncg.edu
%}

%clear all;

%function X = backinduct(var)

tau = tau_opt(1);
gpmmax = tau_opt(2);

%Model Parameters
gas = 2.24; %price in dollars of a gallon of gasoline
%tau = 1.23; %tax in dollars per gallon of gasoline
delta = .96 ; %discount rate between two future periods (long-run)
beta = 0.7; %present-bias discount rate
psi = 2.9; %coefficient of relative risk aversion = -1/price elasticity

taugpm = 0; %tax on gpm

size_mpgvec = 15;

%{
Define scale term in utility function over mileage
Do this for four car types
1: cars (including station wagons)
2: vans
3: SUVs
4: pickups
%}

load caroptions; 
%This loads a 7by4 matrix called car_option
%car_option(j,1) is the car's mpg
%car_option(j,2) is the car's price
%car_option(j,3) is the car's type (1=car/wagon,2=van,3=SUV,4=pickup)
%car_option(j,4) is the car's hybrid status (1=hybrid,2=non-hybrid)
C(1) = (gas+.48)/25*(11681^psi); 
C(2) = (gas+.48)/car_option(2,1)*(13522^psi);
C(3) = (gas+.48)/car_option(3,1)*(14054^psi);
C(4) = (gas+.48)/car_option(4,1)*(12444^psi);

D(1) = 321.4682; %Disutility from car age, found in earlier calibration
D(2) = 578.2071;
D(3) = 614.5608;
D(4) = 204.6875;

T = 90; %maximum age of consumer
first_t = 21; %lowest age of consumer

%Parameterize functions
u = @(m,v,type) (C(type)*m^(1-psi)/(1-psi)) - D(type)*v; %Utility from driving m miles in v year old car
m_star = @(v,gpm,netprice,type) (((gpm*netprice)/C(type))^(-1/psi)); %optimal choice of miles
if psi ~= 1
    u_tilde = @(v,gpm,netprice,type) C(type)*(m_star(v,gpm,netprice,type))^(1-psi)/(1-psi) ...
        - D(type)*v - gpm*netprice*m_star(v,gpm,netprice,type); %indirect utility
else
    u_tilde = @(v,gpm,netprice,type) C(type)*log(m_star(v,gpm,netprice,type))...
        - D(type)*v - gpm*netprice*m_star(v,gpm,netprice,type);
end;

d = @(gc) 1.23*gc; %damage function from emissions from gas consumption; 
    %Set so that marginal damages from gallon of gas = $1.23
    
%Choice of cars

%Fit a curve to the car choices from car options
c0 = zeros(4,1); c1 = zeros(4,1); c2 = zeros(4,1); %initialize
for i = 1:4 %For each of the four types
    x1 = 1/car_option(i,1); y1 = car_option(i,2);
    x2 = 1/car_option(i+4,1); y2 = car_option(i+4,2);
    c2_max = (y2-y1)/(x1-x2)^2; %The maximum value for the curvature of the cost function
    c2(i) = c2_max; %Choose the curvature of the cost curve
    c1(i) = (1/(x1-x2))*(y1-y2-c2(i)*(x1^2-x2^2));
    c0(i) = y1-c1(i)*x1-c2(i)*x1^2;
end;
%Replace cost curve for vans using curve for SUVs
c2(2) = c2(3); c1(2) = c1(3);
c0(2) = c0(3) + car_option(3,2) - car_option(2,2); %vertical shift

c = @(gpm,type) (c0(type) + c1(type)*gpm + c2(type)*gpm.^2); %cost of new car with fuel economy gpm
c_prime = @(gpm,type) (c1(type) + 2*c2(type)*gpm); %marginal cost of gpm
%clear x1 x2 y1 y2 c2_max c2 c1 c0;    

%Calculate utilty cost to individual based on financing
rate = 0.07; %annual interest rate
downpaymentpercent = 0.1; %Fraction of total cost down
loanlength = 6; %period of loan in years
[~,~,~,Payment] = amortize(rate/12,loanlength*12,1-downpaymentpercent);
%Payment is the monthly payment on this loan for a $1 car
Payment = 12*Payment; %Aggregate to annual
loancost = zeros(loanlength,1);
loancost(1) = downpaymentpercent+Payment;
for year = 2:loanlength
    loancost(year) = beta*delta^(year-1)*Payment;
end;
totalloancost = sum(loancost);
%Multiply this by the sticker price of the car
c = @(gpm,type) totalloancost*c(gpm,type);
clear loancost Payment year;

%Discretize states
age_vec = (1:1:40)' ;  %Possible ages that cars can be, no car can be older than this max
%Allowable values for mpg - different for different car types, so use a
%matrix
mpg_vec = zeros(size_mpgvec,4);

mpg_vec(:,1) = linspace(1/gpmmax,33,size_mpgvec)';
mpg_vec(:,2) = linspace(1/gpmmax,21.5,size_mpgvec)';
mpg_vec(:,3) = linspace(1/gpmmax,22,size_mpgvec)';
mpg_vec(:,4) = linspace(1/gpmmax,19.5,size_mpgvec)';

gpm_vec = 1./mpg_vec; %Values in gpm


%Define choice space
x_vec = [gpm_vec;zeros(1,4)]; 
%The last element in each column reflects not scrapping your car

%Initialize the policy function
opt_choice = zeros(length(age_vec),length(gpm_vec),T,4);
%Initialize the value function
value = zeros(length(age_vec),length(gpm_vec),T,4); 

%Define the present-period utility
present_utility = zeros(length(age_vec),length(gpm_vec),length(x_vec),4);
for type = 1:4
for age = 1:length(age_vec)
    for car_i = 1:length(gpm_vec)
        gpm = gpm_vec(car_i,type); %car_i is the index; gpm is the value
        for x_i = 1:length(x_vec)
            x = x_vec(x_i,type);
            if x == 0
                present_utility(age,car_i,x_i,type) = u_tilde(age,gpm,gas+tau,type);
            else
                present_utility(age,car_i,x_i,type) = u_tilde(age,gpm,gas+tau,type)... 
                    - c(x,type) - taugpm*x;
            end;
        end;
    end;
end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Solve through backwards induction

for type = 1:4
for t_i = 1:T-first_t+1
    t = T-t_i+1; %loop through each period, backwards
    %Loop through each state (age and gpm)
    for age = 1:length(age_vec)
        for car_i = 1:length(gpm_vec)
            gpm = gpm_vec(car_i,type); %car_i is the index; gpm is the value
            
            %Loop through each choice (x) and define the utility that you
            %would get at each choice
            u_t = zeros(length(x_vec),t_i); %This will be the discounted utility in the
                                            %present (t) and each future period
            U_t = zeros(length(x_vec),1); %This will be the sum of utilities from 
                                            %present plus all future
                                            %periods
           
            if t == T %in the final period, there are no future periods!
                for x_i = 1:length(x_vec)
                    u_t(x_i) = present_utility(age,car_i,x_i,type);
                    U_t(x_i) = u_t(x_i);
                end;
            elseif t ~= T 
                for x_i = 1:length(x_vec)
                    x = x_vec(x_i,type);
                    u_t(x_i,t) = present_utility(age,car_i,x_i,type); %utility in the present period (t)
                    if x == 0
                        age_next = min(age+1,length(age_vec));
                        car_i_next = car_i;
                    else
                        age_next = 1;
                        car_i_next = x_i;
                    end;
                    
                    %Define the matrix "state" to tell you the state that
                    %you will be in in all future periods, given that your
                    %future selves will be optimizing in each period.
                    %Their optimizing behavior is given by the matrix
                    %"opt_choice."  This is defined for future periods
                    %below, but since we are looping over time backwards,
                    %it will already be defined when it is needed.
                    state = zeros(2,T);
                    %Next period's (t+1) state is determined by your
                    %present (t) decision
                    state(1,t+1) = age_next; 
                    state(2,t+1) = car_i_next;
                    %Utility next period determined by your state next
                    %period and the optimal choice that your next period
                    %self will make, opt_choice(.,.,t+1)
                    u_t(x_i,t+1) = beta*delta*...
                        present_utility(age_next,car_i_next,opt_choice(age_next,car_i_next,t+1,type),type);
                    %For any periods beyond the next period (t+2 or
                    %greater), the state and utility are defined by choices
                    %of your selves even farther in the future.
                    if t < T-1
                        for t_i_2 = t+1:T-1 %use a new time index, t_i_2, to 
                                            %index the future periods for
                                            %self at time t
                            if opt_choice(state(1,t_i_2),state(2,t_i_2),t_i_2,type) == length(x_vec)
                                state(1,t_i_2+1) = min(state(1,t_i_2)+1,length(age_vec));
                                state(2,t_i_2+1) = state(2,t_i_2);
                            else
                                state(1,t_i_2+1) = 1;
                                state(2,t_i_2+1) = opt_choice(state(1,t_i_2),state(2,t_i_2),t_i_2,type);
                            end;
                            %Now we know what utility your t_i_2+1 self
                            %will have
                            u_t(x_i,t_i_2+1) = beta*delta^(t_i_2+1-t)*...
                                present_utility(state(1,t_i_2+1),state(2,t_i_2+1),...
                                opt_choice(state(1,t_i_2+1),state(2,t_i_2+1),t_i_2+1,type),type);
                        end;
                    end;
                    %Sum of all of the u_t's from present and future
                    %selves, for your overall discounted utility from
                    %choice x_i
                    U_t(x_i) = sum(u_t(x_i,:)); 
                end;
            end;
               
            %Find the choice offering the highest utility, and the value of
            %utility
            [value(age,car_i,t,type),opt_choice(age,car_i,t,type)] = max(U_t);
        end;
    end;
end;
end;

%{
%I can check if there is variance in what model is chosen
min(min(min(opt_choice(:,:,21:T,1))));
for type = 1:4
for t_i = 1:T-first_t+1
    t = T-t_i+1; %loop through each period, backwards
    %Loop through each state (age and gpm)
    for age = 1:length(age_vec)
        for car_i = 1:length(gpm_vec)
            if opt_choice(age,car_i,t,type) == length(x_vec)
                opt_choice(age,car_i,t,type) = 0;
            end;
        end;
    end;
end;
end;
max(max(max(opt_choice(:,:,:,1))));
%}


%%%%%%%%%%%%%%%%%%%%%%%%%%%
%The consumer's choice is solved.
%Evaluate true utility (including emissions damages) by simulating for a
%bunch of consumers

N = 100; %Number of consumers of each type

%Initialize state
%state(1,t,i,type) is the age of the car of the i-th consumer of type in period t
%state(2,t,i,type) is the fuel economy state of the car of the i-th consumer of type in period t
state = zeros(2,T,N,4);

%Initialize true utility
true_utility = zeros(T,N,4);
miles_driven = zeros(T,N,4);
gas_consumption = zeros(T,N,4);

%Initial conditions 
%Import age distribution of fleet from NHTS
caragetile = zeros(N,4);
caragetile(:,1) = importdata('caragetile.csv',',',0);
caragetile(:,2) = importdata('vanagetile.csv',',',0);
caragetile(:,3) = importdata('suvagetile.csv',',',0);
caragetile(:,4) = importdata('trkagetile.csv',',',0);

%Import a random seed to generate the initial distribution of fuel
%economies
load randomseed; 
for i = 1:N
    for type = 1:4
        state(1,first_t,i,type) = caragetile(i,type); %The age of the car
        state(2,first_t,i,type) = ceil(randomseed(i)*length(gpm_vec)); %The fuel economy state
    end;
end;


for i = 1:N
    for type = 1:4
        for t = first_t:T
            miles_driven(t,i,type) = m_star(state(1,t,i,type),gpm_vec(state(2,t,i,type),type),gas+tau,type);
            gas_consumption(t,i,type) = miles_driven(t,i,type)*gpm_vec(state(2,t,i,type),type);
            x_i = opt_choice(state(1,t,i,type),state(2,t,i,type),t,type);
            true_utility(t,i,type) = delta^t*(u_tilde(state(1,t,i,type),gpm_vec(state(2,t,i,type),type),gas+tau,type)...
                - d(gpm_vec(state(2,t,i,type),type)*m_star(state(1,t,i,type),gpm_vec(state(2,t,i,type),type),gas+tau,type))...
                + tau*gas_consumption(t,i,type) + taugpm*x_vec(x_i,type));
        
            if x_i == length(x_vec)
                state(1,t+1,i,type) = min(state(1,t,i,type)+1,length(age_vec));
                state(2,t+1,i,type) = state(2,t,i,type);
            else
                state(1,t+1,i,type) = 1;
                state(2,t+1,i,type) = x_i;
            end;
        end;
    end;
end;

avg_car_age = zeros(4,1); avg_miles_driven = zeros(4,1);
sum_true_utility = zeros(4,1); avg_gas_cons = zeros(4,1); avg_gpm = zeros(4,1);
for type = 1:4
    avg_car_age(type) = mean(mean(state(1,first_t:T,:,type)));
    avg_miles_driven(type) = mean(mean(miles_driven(first_t:T,:,type)));
    sum_true_utility(type) = sum(sum(true_utility(:,:,type)));
    avg_gas_cons(type) = mean(mean(gas_consumption(first_t:T,:,type)));
    avg_gpm(type) = mean(mean(gpm_vec(state(2,first_t:T,:,type),type)));
end;

%Calculate weighted sums
%Weight by market share of cars in NHTS 
mkt_share = ([0.5891,0.0939,0.1250,0.1921]);

weighted_car_age = dot(mkt_share,avg_car_age);
weighted_sum_true_utility = dot(mkt_share,sum_true_utility);
weighted_miles = dot(mkt_share,avg_miles_driven);
weighted_gascons = dot(mkt_share,avg_gas_cons);
weighted_gpm = dot(mkt_share,avg_gpm);


load fb; %this contains fb_utility
dwl = (fb_utility - weighted_sum_true_utility)/N;

%To present the results in a table
tablerow(1,1) = dwl;
tablerow(2,1) = weighted_car_age;
tablerow(3,1) = weighted_miles;
tablerow(4,1) = weighted_gascons;
tablerow(5,1) = weighted_gpm;

X=dwl^2;
%}
%}